@@ -2,21 +2,24 @@
 
 use tor_rtcompat::Runtime;
 
+use crate::config::TorRelayConfig;
 use crate::err::Error;
-use crate::{config::TorRelayConfig, TorRelay};
+use crate::relay::TorRelay;
 
 /// An object for constructing a [`TorRelay`].
 ///
 /// Returned by [`TorRelay::builder()`].
 #[derive(Clone)]
 #[must_use]
-pub struct TorRelayBuilder<R: Runtime> {
+#[allow(unused)] // TODO: Remove me when used.
+pub(crate) struct TorRelayBuilder<R: Runtime> {
     /// The runtime for the client to use
     runtime: R,
     /// The configuration.
     config: TorRelayConfig,
 }
 
+#[allow(unused)] // TODO: Remove me when used.
 impl<R: Runtime> TorRelayBuilder<R> {
     /// Construct a new TorClientBuilder with the given runtime.
     pub(crate) fn new(runtime: R) -> Self {
@@ -27,7 +30,7 @@ impl<R: Runtime> TorRelayBuilder<R> {
     }
 
     /// Return a newly created TorRelay object.
-    pub fn create(&self) -> Result<TorRelay<R>, Error> {
+    pub(crate) fn create(&self) -> Result<TorRelay<R>, Error> {
         TorRelay::create_inner(self.runtime.clone(), &self.config).map_err(Into::into)
     }
 }
