@@ -41,6 +41,8 @@
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
+// TODO #1645 (either remove this, or decide to have it everywhere)
+#![cfg_attr(not(all(feature = "full", feature = "experimental")), allow(unused))]
 // Overrides specific to this crate:
 #![allow(clippy::print_stderr)]
 #![allow(clippy::print_stdout)]
@@ -113,7 +115,7 @@ use clap::{value_parser, Arg, ArgAction, Command};
 #[allow(unused_imports)]
 use tracing::{error, info, warn};
 
-#[cfg(any(feature = "hsc", feature = "onion-service-service", feature = "relay"))]
+#[cfg(any(feature = "hsc", feature = "onion-service-service"))]
 use clap::Subcommand as _;
 
 #[cfg(feature = "experimental-api")]
@@ -296,12 +298,6 @@ where
         }
     }
 
-    cfg_if::cfg_if! {
-        if #[cfg(feature = "relay")] {
-            let clap_app = subcommands::relay::RelaySubcommands::augment_subcommands(clap_app);
-        }
-    }
-
     // Tracing doesn't log anything when there is no subscriber set.  But we want to see
     // logging messages from config parsing etc.  We can't set the global default subscriber
     // because we can only set it once.  The other ways involve a closure.  So we have a
@@ -409,15 +405,6 @@ where
         }
     }
 
-    // Check for the optional "relay" subcommand.
-    cfg_if::cfg_if! {
-        if #[cfg(feature = "relay")] {
-            if let Some(relay_matches) = matches.subcommand_matches("relay") {
-                return subcommands::relay::run(runtime, relay_matches, &config);
-            }
-        }
-    }
-
     panic!("Subcommand added to clap subcommand list, but not yet implemented");
 }
 
