@@ -1,21 +1,22 @@
 //! Abstract implementation of a channel manager
 
-use crate::mgr::state::{OpenEntry, PendingEntry};
+use crate::mgr::state::{ChannelForTarget, PendingChannelHandle};
 use crate::{ChanProvenance, ChannelConfig, ChannelUsage, Dormancy, Error, Result};
 
 use crate::factory::BootstrapReporter;
 use async_trait::async_trait;
-use futures::future::{FutureExt, Shared};
+use futures::future::Shared;
 use oneshot_fused_workaround as oneshot;
 use std::result::Result as StdResult;
 use std::sync::Arc;
 use std::time::Duration;
-use tor_basic_utils::RngExt as _;
-use tor_error::internal;
-use tor_linkspec::{HasRelayIds, RelayIds};
+use tor_error::{error_report, internal};
+use tor_linkspec::HasRelayIds;
 use tor_netdir::params::NetParameters;
 use tor_proto::channel::params::ChannelPaddingInstructionsUpdates;
+use tor_proto::memquota::{ChannelAccount, SpecificAccount as _, ToplevelAccount};
 
+mod select;
 mod state;
 
 /// Trait to describe as much of a
@@ -73,6 +74,7 @@ pub(crate) trait AbstractChannelFactory {
         &self,
         target: &Self::BuildSpec,
         reporter: BootstrapReporter,
+        memquota: ChannelAccount,
     ) -> Result<Arc<Self::Channel>>;
 
     /// Construct a new channel for an incoming connection.
@@ -81,6 +83,7 @@ pub(crate) trait AbstractChannelFactory {
         &self,
         peer: std::net::SocketAddr,
         stream: Self::Stream,
+        memquota: ChannelAccount,
     ) -> Result<Arc<Self::Channel>>;
 }
 
@@ -101,6 +104,9 @@ pub(crate) struct AbstractChanMgr<CF: AbstractChannelFactory> {
 
     /// A bootstrap reporter to give out when building channels.
     pub(crate) reporter: BootstrapReporter,
+
+    /// The memory quota account that every channel will be a child of
+    pub(crate) memquota: ToplevelAccount,
 }
 
 /// Type alias for a future that we wait on to see when a pending
@@ -119,10 +125,12 @@ impl<CF: AbstractChannelFactory + Clone> AbstractChanMgr<CF> {
         dormancy: Dormancy,
         netparams: &NetParameters,
         reporter: BootstrapReporter,
+        memquota: ToplevelAccount,
     ) -> Self {
         AbstractChanMgr {
             channels: state::MgrState::new(connector, config.clone(), dormancy, netparams),
             reporter,
+            memquota,
         }
     }
 
@@ -141,17 +149,6 @@ impl<CF: AbstractChannelFactory + Clone> AbstractChanMgr<CF> {
         self.channels.remove_unusable()
     }
 
-    /// Helper: return the objects used to inform pending tasks
-    /// about a newly open or failed channel.
-    fn setup_launch<C>(&self, ids: RelayIds) -> (state::ChannelState<C>, Sending) {
-        let (snd, rcv) = oneshot::channel();
-        let pending = rcv.shared();
-        (
-            state::ChannelState::Building(state::PendingEntry { ids, pending }),
-            snd,
-        )
-    }
-
     /// Build a channel for an incoming stream. See
     /// [`ChanMgr::handle_incoming`](crate::ChanMgr::handle_incoming).
     #[cfg(feature = "relay")]
@@ -161,8 +158,9 @@ impl<CF: AbstractChannelFactory + Clone> AbstractChanMgr<CF> {
         stream: CF::Stream,
     ) -> Result<Arc<CF::Channel>> {
         let chan_builder = self.channels.builder();
+        let memquota = ChannelAccount::new(&self.memquota)?;
         let _outcome = chan_builder
-            .build_channel_using_incoming(src, stream)
+            .build_channel_using_incoming(src, stream, memquota)
             .await?;
 
         // TODO RELAY: we need to do something with the channel here now that we've created it
@@ -258,26 +256,63 @@ impl<CF: AbstractChannelFactory + Clone> AbstractChanMgr<CF> {
                     }
                 }
                 // We need to launch a channel.
-                Some(Action::Launch(send)) => {
-                    let connector = self.channels.builder();
-                    let outcome = connector
-                        .build_channel(&target, self.reporter.clone())
+                Some(Action::Launch((handle, send))) => {
+                    // WARNING: do not drop the handle without calling
+                    // `upgrade_pending_channel_to_open` or `remove_pending_channel`. If you don't,
+                    // then the pending entry will remain in the channel map forever, and arti will
+                    // be unable to build new channels to the target relay of that pending channel.
+                    //
+                    // This code is ugly since it tries to use IIFE-inspired immediately awaited
+                    // async block expressions to prevent the code from returning early (for example
+                    // due to a `?` operator). When modifying this code, be careful to not add a
+                    // code path that returns early before properly passing `handle` back to
+                    // `MgrState` (see `PendingChannelHandle` for details).
+                    let outcome = async {
+                        let chan_result = async {
+                            let connector = self.channels.builder();
+                            let memquota = ChannelAccount::new(&self.memquota)?;
+
+                            connector
+                                .build_channel(&target, self.reporter.clone(), memquota)
+                                .await
+                        }
                         .await;
-                    let status = self.handle_build_outcome(&target, outcome);
+
+                        match chan_result {
+                            Ok(ref chan) => {
+                                // Replace the pending channel with the newly built channel.
+                                self.channels
+                                    .upgrade_pending_channel_to_open(handle, Arc::clone(chan))?;
+                            }
+                            Err(_) => {
+                                // Remove the pending channel.
+                                if let Err(e) = self.channels.remove_pending_channel(handle) {
+                                    // Just log an error if we're unable to remove it, since there's
+                                    // nothing else we can do here, and returning the error would
+                                    // hide the actual error that we care about (the channel build
+                                    // failure). Panic in debug builds so that we catch this error
+                                    // sooner.
+                                    #[allow(clippy::missing_docs_in_private_items)]
+                                    const MSG: &str = "Unable to remove the pending channel";
+                                    error_report!(internal!("{e}"), "{}", MSG);
+                                    #[cfg(debug_assertions)]
+                                    panic!("{MSG}: {e}");
+                                }
+                            }
+                        }
+
+                        chan_result
+                    }
+                    .await;
 
                     // It's okay if all the receivers went away:
                     // that means that nobody was waiting for this channel.
-                    let _ignore_err = send.send(status.clone().map(|_| ()));
+                    let _ignore_err = send.send(outcome.clone().map(|_| ()));
 
-                    match status {
-                        Ok(Some(chan)) => {
+                    match outcome {
+                        Ok(chan) => {
                             return Ok((chan, ChanProvenance::NewlyCreated));
                         }
-                        Ok(None) => {
-                            final_attempt = true;
-                            provenance = ChanProvenance::NewlyCreated;
-                            last_err.get_or_insert(Error::RequestCancelled);
-                        }
                         Err(e) => last_err = Some(e),
                     }
                 }
@@ -302,176 +337,29 @@ impl<CF: AbstractChannelFactory + Clone> AbstractChanMgr<CF> {
         target: &CF::BuildSpec,
         final_attempt: bool,
     ) -> Result<Option<Action<CF::Channel>>> {
-        use state::ChannelState::*;
-        self.channels.with_channels(|channel_map| {
-            match channel_map.by_all_ids(target) {
-                Some(Open(OpenEntry { channel, .. })) => {
-                    if channel.is_usable() {
-                        // This entry is a perfect match for the target keys:
-                        // we'll return the open entry.
-                        return Ok(Some(Action::Return(Ok(channel.clone()))));
-                    } else if final_attempt {
-                        // We don't launch an attempt in this case.
-                        return Ok(None);
-                    } else {
-                        // This entry was a perfect match for the target, but it
-                        // is no longer usable! We launch a new connection to
-                        // this target, and wait on that.
-                        let (new_state, send) = self.setup_launch(RelayIds::from_relay_ids(target));
-                        channel_map.try_insert(new_state)?;
+        // don't create new channels on the final attempt
+        let response = self.channels.request_channel(
+            target,
+            /* add_new_entry_if_not_found= */ !final_attempt,
+        );
 
-                        return Ok(Some(Action::Launch(send)));
-                    }
-                }
-                Some(Building(PendingEntry { pending, .. })) => {
-                    // This entry is a perfect match for the target keys: we'll
-                    // return the pending entry. (We don't know for sure if it
-                    // will match once it completes, since we might discover
-                    // additional keys beyond those listed for this pending
-                    // entry.)
-                    if final_attempt {
-                        // We don't launch an attempt in this case.
-                        return Ok(None);
-                    }
-                    return Ok(Some(Action::Wait(pending.clone())));
-                }
-                _ => {}
-            }
-            // Okay, we don't have an exact match.  But we might have one or more _partial_ matches?
-            let overlapping = channel_map.all_overlapping(target);
-            if overlapping
-                .iter()
-                .any(|entry| matches!(entry, Open(OpenEntry{ channel, ..}) if channel.is_usable()))
-            {
-                // At least one *open, usable* channel has been negotiated that
-                // overlaps only partially with our target: it has proven itself
-                // to have _one_ of our target identities, but not all.
-                //
-                // Because this channel exists, we know that our target cannot
-                // succeed, since relays are not allowed to share _any_
-                // identities.
-                return Ok(Some(Action::Return(Err(Error::IdentityConflict))));
-            } else if let Some(first_building) = overlapping
-                .iter()
-                .find(|entry| matches!(entry, Building(_)))
-            {
-                // There is at least one *in-progress* channel that has at least
-                // one identity in common with our target, but it does not have
-                // _all_ the identities we want.
-                //
-                // If it succeeds, we might find that we can use it; or we might
-                // find out that it is not suitable.  So we'll wait for it, and
-                // see what happens.
-                //
-                // TODO: This approach will _not_ be sufficient once  we are
-                // implementing relays, and some of our channels are created in
-                // response to client requests.
-                match first_building {
-                    Open(_) => unreachable!(),
-                    Building(PendingEntry { pending, .. }) => {
-                        if final_attempt {
-                            // We don't wait in this case.
-                            return Ok(None);
-                        }
-                        return Ok(Some(Action::Wait(pending.clone())));
-                    }
+        match response {
+            Ok(Some(ChannelForTarget::Open(channel))) => Ok(Some(Action::Return(Ok(channel)))),
+            Ok(Some(ChannelForTarget::Pending(pending))) => {
+                if !final_attempt {
+                    Ok(Some(Action::Wait(pending)))
+                } else {
+                    // don't return a pending channel on the final attempt
+                    Ok(None)
                 }
             }
-
-            if final_attempt {
-                // We don't launch an attempt in this case.
-                return Ok(None);
-            }
-
-            // Great, nothing interfered at all.
-            let (new_state, send) = self.setup_launch(RelayIds::from_relay_ids(target));
-            channel_map.try_insert(new_state)?;
-            Ok(Some(Action::Launch(send)))
-        })?
-    }
-
-    /// We just tried to build a channel: Handle the outcome and decide what to
-    /// do.
-    ///
-    /// Return `Ok(None)` if we have a transient error that we expect will be
-    /// cleaned up by one final call to `choose_action`.
-    fn handle_build_outcome(
-        &self,
-        target: &CF::BuildSpec,
-        outcome: Result<Arc<CF::Channel>>,
-    ) -> Result<Option<Arc<CF::Channel>>> {
-        use state::ChannelState::*;
-        match outcome {
-            Ok(chan) => {
-                // The channel got built: remember it, tell the
-                // others, and return it.
-                self.channels
-                    .with_channels_and_params(|channel_map, channels_params| {
-                        match channel_map.remove_exact(target) {
-                            Some(Building(_)) => {
-                                // We successfully removed our pending
-                                // action. great!  Fall through and add
-                                // the channel we just built.
-                            }
-                            None => {
-                                // Something removed our entry from the list.
-                                // Time to retry.
-                                return Ok(None);
-                            }
-                            Some(ent @ Open(_)) => {
-                                // Oh no. Something else built an entry
-                                // here, and replaced us.  Put that
-                                // something back, and retry.
-
-                                channel_map.insert(ent);
-                                return Ok(None);
-                            }
-                        }
-
-                        // This isn't great.  We context switch to the newly-created
-                        // channel just to tell it how and whether to do padding.  Ideally
-                        // we would pass the params at some suitable point during
-                        // building.  However, that would involve the channel taking a
-                        // copy of the params, and that must happen in the same channel
-                        // manager lock acquisition span as the one where we insert the
-                        // channel into the table so it will receive updates.  I.e.,
-                        // here.
-                        let update = channels_params.initial_update();
-                        if let Some(update) = update {
-                            chan.reparameterize(update.into())
-                                .map_err(|_| internal!("failure on new channel"))?;
-                        }
-                        let new_entry = Open(OpenEntry {
-                            channel: chan.clone(),
-                            max_unused_duration: Duration::from_secs(
-                                rand::thread_rng()
-                                    .gen_range_checked(180..270)
-                                    .expect("not 180 < 270 !"),
-                            ),
-                        });
-                        channel_map.insert(new_entry);
-                        Ok(Some(chan))
-                    })?
-            }
-            Err(e) => {
-                // The channel failed. Make it non-pending, tell the
-                // others, and set the error.
-                self.channels.with_channels(|channel_map| {
-                    match channel_map.remove_exact(target) {
-                        Some(Building(_)) | None => {
-                            // We successfully removed our pending
-                            // action, or somebody else did.
-                        }
-                        Some(ent @ Open(_)) => {
-                            // Oh no. Something else built an entry
-                            // here, and replaced us.  Put that
-                            // something back.
-                            channel_map.insert(ent);
-                        }
-                    }
-                })?;
-                Err(e)
+            Ok(Some(ChannelForTarget::NewEntry((handle, send)))) => {
+                // do not drop the handle if refactoring; see `PendingChannelHandle` for details
+                Ok(Some(Action::Launch((handle, send))))
             }
+            Ok(None) => Ok(None),
+            Err(e @ Error::IdentityConflict) => Ok(Some(Action::Return(Err(e)))),
+            Err(e) => Err(e),
         }
     }
 
@@ -515,18 +403,22 @@ impl<CF: AbstractChannelFactory + Clone> AbstractChanMgr<CF> {
         self.channels.expire_channels()
     }
 
-    /// Test only: return the current open usable channel with a given
-    /// `ident`, if any.
+    /// Test only: return the open usable channels with a given `ident`.
     #[cfg(test)]
-    pub(crate) fn get_nowait<'a, T>(&self, ident: T) -> Option<Arc<CF::Channel>>
+    pub(crate) fn get_nowait<'a, T>(&self, ident: T) -> Vec<Arc<CF::Channel>>
     where
         T: Into<tor_linkspec::RelayIdRef<'a>>,
     {
         use state::ChannelState::*;
         self.channels
-            .with_channels(|channel_map| match channel_map.by_id(ident) {
-                Some(Open(ref ent)) if ent.channel.is_usable() => Some(ent.channel.clone()),
-                _ => None,
+            .with_channels(|channel_map| {
+                channel_map
+                    .by_id(ident)
+                    .filter_map(|entry| match entry {
+                        Open(ref ent) if ent.channel.is_usable() => Some(Arc::clone(&ent.channel)),
+                        _ => None,
+                    })
+                    .collect()
             })
             .expect("Poisoned lock")
     }
@@ -534,10 +426,10 @@ impl<CF: AbstractChannelFactory + Clone> AbstractChanMgr<CF> {
 
 /// Possible actions that we'll decide to take when asked for a channel.
 #[allow(clippy::large_enum_variant)]
-enum Action<C> {
+enum Action<C: AbstractChannel> {
     /// We found no channel.  We're going to launch a new one,
     /// then tell everybody about it.
-    Launch(Sending),
+    Launch((PendingChannelHandle, Sending)),
     /// We found an in-progress attempt at making a channel.
     /// We're going to wait for it to finish.
     Wait(Pending),
@@ -569,6 +461,7 @@ mod test {
     use std::time::Duration;
     use tor_error::bad_api_usage;
     use tor_llcrypto::pk::ed25519::Ed25519Identity;
+    use tor_memquota::ArcMemoryQuotaTrackerExt as _;
 
     use crate::ChannelUsage as CU;
     use tor_rtcompat::{task::yield_now, test_with_one_runtime, Runtime};
@@ -642,6 +535,7 @@ mod test {
             Default::default(),
             &Default::default(),
             BootstrapReporter::fake(),
+            ToplevelAccount::new_noop(),
         )
     }
 
@@ -677,6 +571,7 @@ mod test {
             &self,
             target: &Self::BuildSpec,
             _reporter: BootstrapReporter,
+            _memquota: ChannelAccount,
         ) -> Result<Arc<FakeChannel>> {
             yield_now().await;
             let FakeBuildSpec(ident, mood, id) = *target;
@@ -705,6 +600,7 @@ mod test {
             &self,
             _peer: std::net::SocketAddr,
             _stream: Self::Stream,
+            _memquota: ChannelAccount,
         ) -> Result<Arc<Self::Channel>> {
             unimplemented!()
         }
@@ -723,9 +619,7 @@ mod test {
             let chan2 = mgr.get_or_launch(target, CU::UserTraffic).await.unwrap().0;
 
             assert_eq!(chan1, chan2);
-
-            let chan3 = mgr.get_nowait(&u32_to_ed(413)).unwrap();
-            assert_eq!(chan1, chan3);
+            assert_eq!(mgr.get_nowait(&u32_to_ed(413)), vec![chan1]);
         });
     }
 
@@ -739,8 +633,7 @@ mod test {
             let res1 = mgr.get_or_launch(target, CU::UserTraffic).await;
             assert!(matches!(res1, Err(Error::UnusableTarget(_))));
 
-            let chan3 = mgr.get_nowait(&u32_to_ed(999));
-            assert!(chan3.is_none());
+            assert!(mgr.get_nowait(&u32_to_ed(999)).is_empty());
         });
     }
 
@@ -804,9 +697,9 @@ mod test {
 
             mgr.remove_unusable_entries().unwrap();
 
-            assert!(mgr.get_nowait(&u32_to_ed(3)).is_some());
-            assert!(mgr.get_nowait(&u32_to_ed(4)).is_some());
-            assert!(mgr.get_nowait(&u32_to_ed(5)).is_none());
+            assert!(!mgr.get_nowait(&u32_to_ed(3)).is_empty());
+            assert!(!mgr.get_nowait(&u32_to_ed(4)).is_empty());
+            assert!(mgr.get_nowait(&u32_to_ed(5)).is_empty());
         });
     }
 }
