@@ -3,17 +3,19 @@
 use std::time::Duration;
 
 use super::AbstractChannelFactory;
-use super::{AbstractChannel, Pending};
-use crate::{ChannelConfig, Dormancy, Result};
+use super::{select, AbstractChannel, Pending, Sending};
+use crate::{ChannelConfig, Dormancy, Error, Result};
 
+use futures::FutureExt;
 use std::result::Result as StdResult;
+use std::sync::atomic::{AtomicU64, Ordering};
 use std::sync::Arc;
+use tor_async_utils::oneshot;
+use tor_basic_utils::RngExt as _;
 use tor_cell::chancell::msg::PaddingNegotiate;
 use tor_config::PaddingLevel;
-use tor_error::{internal, into_internal};
-use tor_linkspec::ByRelayIds;
-use tor_linkspec::HasRelayIds;
-use tor_linkspec::RelayIds;
+use tor_error::{error_report, internal, into_internal};
+use tor_linkspec::{HasRelayIds, ListByRelayIds, RelayIds};
 use tor_netdir::{params::NetParameters, params::CHANNEL_PADDING_TIMEOUT_UPPER_BOUND};
 use tor_proto::channel::padding::Parameters as PaddingParameters;
 use tor_proto::channel::padding::ParametersBuilder as PaddingParametersBuilder;
@@ -48,8 +50,8 @@ struct Inner<C: AbstractChannelFactory> {
     /// hand out clones of it when asked.
     builder: C,
 
-    /// A map from identity to channel, or to pending channel status.
-    channels: ByRelayIds<ChannelState<C::Channel>>,
+    /// A map from identity to channels, or to pending channel statuses.
+    channels: ListByRelayIds<ChannelState<C::Channel>>,
 
     /// Parameters for channels that we create, and that all existing channels are using
     ///
@@ -102,6 +104,29 @@ pub(crate) struct OpenEntry<C> {
     pub(crate) max_unused_duration: Duration,
 }
 
+/// A unique ID for a pending ([`PendingEntry`]) channel.
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
+pub(crate) struct UniqPendingChanId(u64);
+
+impl UniqPendingChanId {
+    /// Construct a new `UniqPendingChanId`.
+    pub(crate) fn new() -> Self {
+        /// The next unique ID.
+        static NEXT_ID: AtomicU64 = AtomicU64::new(0);
+        // Relaxed ordering is fine; we don't care about how this
+        // is instantiated with respect to other channels.
+        let id = NEXT_ID.fetch_add(1, Ordering::Relaxed);
+        assert!(id != u64::MAX, "Exhausted the pending channel ID namespace");
+        Self(id)
+    }
+}
+
+impl std::fmt::Display for UniqPendingChanId {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "PendingChan {}", self.0)
+    }
+}
+
 /// An entry for a not-yet-build channel
 #[derive(Clone)]
 pub(crate) struct PendingEntry {
@@ -114,6 +139,9 @@ pub(crate) struct PendingEntry {
     /// This entry will be removed from the map (and possibly replaced with an
     /// `OpenEntry`) _before_ this future becomes ready.
     pub(crate) pending: Pending,
+
+    /// A unique ID that allows us to find this exact pending entry later.
+    pub(crate) unique_id: UniqPendingChanId,
 }
 
 impl<C> HasRelayIds for ChannelState<C>
@@ -242,7 +270,7 @@ impl<C: AbstractChannelFactory> MgrState<C> {
         MgrState {
             inner: std::sync::Mutex::new(Inner {
                 builder,
-                channels: ByRelayIds::new(),
+                channels: ListByRelayIds::new(),
                 config,
                 channels_params,
                 dormancy,
@@ -250,15 +278,22 @@ impl<C: AbstractChannelFactory> MgrState<C> {
         }
     }
 
-    /// Run a function on the `ByRelayIds` that implements the map in this `MgrState`.
+    /// Run a function on the [`ListByRelayIds`] that implements the map in this `MgrState`.
     ///
     /// This function grabs a mutex: do not provide a slow function.
     ///
     /// We provide this function rather than exposing the channels set directly,
     /// to make sure that the calling code doesn't await while holding the lock.
+    ///
+    /// This is only `cfg(test)` since it can deadlock.
+    ///
+    /// # Deadlock
+    ///
+    /// Calling a method on [`MgrState`] from within `func` may cause a deadlock.
+    #[cfg(test)]
     pub(crate) fn with_channels<F, T>(&self, func: F) -> Result<T>
     where
-        F: FnOnce(&mut ByRelayIds<ChannelState<C::Channel>>) -> T,
+        F: FnOnce(&mut ListByRelayIds<ChannelState<C::Channel>>) -> T,
     {
         let mut inner = self.inner.lock()?;
         Ok(func(&mut inner.channels))
@@ -274,6 +309,10 @@ impl<C: AbstractChannelFactory> MgrState<C> {
     }
 
     /// Run a function to modify the builder stored in this state.
+    ///
+    /// # Deadlock
+    ///
+    /// Calling a method on [`MgrState`] from within `func` may cause a deadlock.
     #[allow(dead_code)]
     pub(crate) fn with_mut_builder<F>(&self, func: F)
     where
@@ -283,28 +322,7 @@ impl<C: AbstractChannelFactory> MgrState<C> {
         func(&mut inner.builder);
     }
 
-    /// Run a function on the `ByRelayIds` that implements the map in this `MgrState`.
-    ///
-    /// This function grabs a mutex: do not provide a slow function.
-    ///
-    /// We provide this function rather than exposing the channels set directly,
-    /// to make sure that the calling code doesn't await while holding the lock.
-    pub(crate) fn with_channels_and_params<F, T>(&self, func: F) -> Result<T>
-    where
-        F: FnOnce(&mut ByRelayIds<ChannelState<C::Channel>>, &ChannelPaddingInstructions) -> T,
-    {
-        let mut inner = self.inner.lock()?;
-        // We need this silly destructuring syntax so that we don't seem to be
-        // borrowing the structure mutably and immutably at the same time.
-        let Inner {
-            ref mut channels,
-            ref channels_params,
-            ..
-        } = &mut *inner;
-        Ok(func(channels, channels_params))
-    }
-
-    /// Remove every unusable state from the map in this state..
+    /// Remove every unusable state from the map in this state.
     #[cfg(test)]
     pub(crate) fn remove_unusable(&self) -> Result<()> {
         let mut inner = self.inner.lock()?;
@@ -315,6 +333,153 @@ impl<C: AbstractChannelFactory> MgrState<C> {
         Ok(())
     }
 
+    /// Request an open or pending channel to `target`. If `add_new_entry_if_not_found` is true and
+    /// an open or pending channel isn't found, a new pending entry will be added and
+    /// [`ChannelForTarget::NewEntry`] will be returned. This is all done as part of the same method
+    /// so that all operations are performed under the same lock acquisition.
+    pub(crate) fn request_channel(
+        &self,
+        target: &C::BuildSpec,
+        add_new_entry_if_not_found: bool,
+    ) -> Result<Option<ChannelForTarget<C>>> {
+        use ChannelState::*;
+
+        let mut inner = self.inner.lock()?;
+
+        // The idea here is to choose the channel in two steps:
+        //
+        // - Eligibility: Get channels from the channel map and filter them down to only channels
+        //   which are eligible to be returned.
+        // - Ranking: From the eligible channels, choose the best channel.
+        //
+        // Another way to choose the channel could be something like: first try all canonical open
+        // channels, then all non-canonical open channels, then all pending channels with all
+        // matching relay ids, then remaining pending channels, etc. But this ends up being hard to
+        // follow and inflexible (what if you want to prioritize pending channels over non-canonical
+        // open channels?).
+
+        // Open channels which are allowed for requests to `target`.
+        let open_channels = inner
+            .channels
+            // channels with all target relay identifiers
+            .by_all_ids(target)
+            .filter(|entry| match entry {
+                Open(x) => select::open_channel_is_allowed(x, target),
+                Building(_) => false,
+            });
+
+        // Pending channels which will *probably* be allowed for requests to `target` once they
+        // complete.
+        let pending_channels = inner
+            .channels
+            // channels that have a subset of the relay ids of `target`
+            .all_subset(target)
+            .into_iter()
+            .filter(|entry| match entry {
+                Open(_) => false,
+                Building(x) => select::pending_channel_maybe_allowed(x, target),
+            });
+
+        match select::choose_best_channel(open_channels.chain(pending_channels), target) {
+            Some(Open(OpenEntry { channel, .. })) => {
+                // This entry is a perfect match for the target keys: we'll return the open
+                // entry.
+                return Ok(Some(ChannelForTarget::Open(Arc::clone(channel))));
+            }
+            Some(Building(PendingEntry { pending, .. })) => {
+                // This entry is potentially a match for the target identities: we'll return the
+                // pending entry. (We don't know for sure if it will match once it completes,
+                // since we might discover additional keys beyond those listed for this pending
+                // entry.)
+                return Ok(Some(ChannelForTarget::Pending(pending.clone())));
+            }
+            None => {}
+        }
+
+        // It's possible we know ahead of time that building a channel would be unsuccessful.
+        if inner
+            .channels
+            // channels with at least one id in common with `target`
+            .all_overlapping(target)
+            .into_iter()
+            // but not channels which completely satisfy the id requirements of `target`
+            .filter(|entry| !entry.has_all_relay_ids_from(target))
+            .any(|entry| matches!(entry, Open(OpenEntry{ channel, ..}) if channel.is_usable()))
+        {
+            // At least one *open, usable* channel has been negotiated that overlaps only
+            // partially with our target: it has proven itself to have _one_ of our target
+            // identities, but not all.
+            //
+            // Because this channel exists, we know that our target cannot succeed, since relays
+            // are not allowed to share _any_ identities.
+            //return Ok(Some(Action::Return(Err(Error::IdentityConflict))));
+            return Err(Error::IdentityConflict);
+        }
+
+        if !add_new_entry_if_not_found {
+            return Ok(None);
+        }
+
+        // Great, nothing interfered at all.
+        let any_relay_id = target
+            .identities()
+            .next()
+            .ok_or(internal!("relay target had no id"))?
+            .to_owned();
+        let (new_state, send, unique_id) = setup_launch(RelayIds::from_relay_ids(target));
+        inner
+            .channels
+            .try_insert(ChannelState::Building(new_state))?;
+        let handle = PendingChannelHandle::new(any_relay_id, unique_id);
+        Ok(Some(ChannelForTarget::NewEntry((handle, send))))
+    }
+
+    /// Remove the pending channel identified by its `handle`.
+    pub(crate) fn remove_pending_channel(&self, handle: PendingChannelHandle) -> Result<()> {
+        let mut inner = self.inner.lock()?;
+        remove_pending(&mut inner.channels, handle);
+        Ok(())
+    }
+
+    /// Upgrade the pending channel identified by its `handle` by replacing it with a new open
+    /// `channel`.
+    pub(crate) fn upgrade_pending_channel_to_open(
+        &self,
+        handle: PendingChannelHandle,
+        channel: Arc<C::Channel>,
+    ) -> Result<()> {
+        // Do all operations under the same lock acquisition.
+        let mut inner = self.inner.lock()?;
+
+        remove_pending(&mut inner.channels, handle);
+
+        // This isn't great.  We context switch to the newly-created
+        // channel just to tell it how and whether to do padding.  Ideally
+        // we would pass the params at some suitable point during
+        // building.  However, that would involve the channel taking a
+        // copy of the params, and that must happen in the same channel
+        // manager lock acquisition span as the one where we insert the
+        // channel into the table so it will receive updates.  I.e.,
+        // here.
+        let update = inner.channels_params.initial_update();
+        if let Some(update) = update {
+            channel
+                .reparameterize(update.into())
+                .map_err(|_| internal!("failure on new channel"))?;
+        }
+        let new_entry = ChannelState::Open(OpenEntry {
+            channel,
+            max_unused_duration: Duration::from_secs(
+                rand::thread_rng()
+                    .gen_range_checked(180..270)
+                    .expect("not 180 < 270 !"),
+            ),
+        });
+        inner.channels.insert(new_entry);
+
+        Ok(())
+    }
+
     /// Reconfigure all channels as necessary
     ///
     /// (By reparameterizing channels as needed)
@@ -393,6 +558,97 @@ impl<C: AbstractChannelFactory> MgrState<C> {
     }
 }
 
+/// A channel for a given target relay.
+pub(crate) enum ChannelForTarget<CF: AbstractChannelFactory> {
+    /// A channel that is open.
+    Open(Arc<CF::Channel>),
+    /// A channel that is building.
+    Pending(Pending),
+    /// Information about a new pending channel entry.
+    NewEntry((PendingChannelHandle, Sending)),
+}
+
+/// A handle for a pending channel.
+///
+/// WARNING: This handle should never be dropped, and should always be passed back into
+/// [`MgrState::remove_pending_channel`] or [`MgrState::upgrade_pending_channel_to_open`], otherwise
+/// the pending channel may be left in the channel map forever.
+///
+/// This handle must only be used with the `MgrState` from which it was given.
+pub(crate) struct PendingChannelHandle {
+    /// Any relay ID for this pending channel.
+    relay_id: tor_linkspec::RelayId,
+    /// The unique ID for this pending channel.
+    unique_id: UniqPendingChanId,
+    /// The pending channel has been removed from the channel map.
+    chan_has_been_removed: bool,
+}
+
+impl PendingChannelHandle {
+    /// Create a new [`PendingChannelHandle`].
+    fn new(relay_id: tor_linkspec::RelayId, unique_id: UniqPendingChanId) -> Self {
+        Self {
+            relay_id,
+            unique_id,
+            chan_has_been_removed: false,
+        }
+    }
+
+    /// This should be called when the pending channel has been removed from the pending channel
+    /// map. Not calling this will result in an error log message (and panic in debug builds) when
+    /// this handle is dropped.
+    fn chan_has_been_removed(mut self) {
+        self.chan_has_been_removed = true;
+    }
+}
+
+impl std::ops::Drop for PendingChannelHandle {
+    fn drop(&mut self) {
+        if !self.chan_has_been_removed {
+            // log, and also panic in debug builds
+            #[allow(clippy::missing_docs_in_private_items)]
+            const MSG: &str = "Dropped the 'PendingChannelHandle' without removing the channel";
+            error_report!(
+                internal!("{MSG}"),
+                "'PendingChannelHandle' dropped unexpectedly",
+            );
+            #[cfg(debug_assertions)]
+            panic!("{MSG}");
+        }
+    }
+}
+
+/// Helper: return the objects used to inform pending tasks about a newly open or failed channel.
+fn setup_launch(ids: RelayIds) -> (PendingEntry, Sending, UniqPendingChanId) {
+    let (snd, rcv) = oneshot::channel();
+    let pending = rcv.shared();
+    let unique_id = UniqPendingChanId::new();
+    let entry = PendingEntry {
+        ids,
+        pending,
+        unique_id,
+    };
+
+    (entry, snd, unique_id)
+}
+
+/// Helper: remove the pending channel identified by `handle` from `channel_map`.
+fn remove_pending<C: AbstractChannel>(
+    channel_map: &mut tor_linkspec::ListByRelayIds<ChannelState<C>>,
+    handle: PendingChannelHandle,
+) {
+    // we need only one relay id to locate it, even if it has multiple relay ids
+    let removed = channel_map.remove_by_id(&handle.relay_id, |c| {
+        let ChannelState::Building(c) = c else {
+            return false;
+        };
+        c.unique_id == handle.unique_id
+    });
+    debug_assert_eq!(removed.len(), 1, "expected to remove exactly one channel");
+
+    handle.chan_has_been_removed();
+}
+
 /// Converts config, dormancy, and netdir, into parameter updates
 ///
 /// Calculates new parameters, updating `channels_params` as appropriate.
@@ -547,6 +803,7 @@ mod test {
     use std::sync::{Arc, Mutex};
     use tor_llcrypto::pk::ed25519::Ed25519Identity;
     use tor_proto::channel::params::ChannelPaddingInstructionsUpdates;
+    use tor_proto::memquota::ChannelAccount;
 
     fn new_test_state() -> MgrState<FakeChannelFactory> {
         MgrState::new(
@@ -573,6 +830,7 @@ mod test {
             &self,
             _target: &Self::BuildSpec,
             _reporter: BootstrapReporter,
+            _memquota: ChannelAccount,
         ) -> Result<Arc<FakeChannel>> {
             unimplemented!()
         }
@@ -582,6 +840,7 @@ mod test {
             &self,
             _peer: std::net::SocketAddr,
             _stream: Self::Stream,
+            _memquota: ChannelAccount,
         ) -> Result<Arc<Self::Channel>> {
             unimplemented!()
         }
@@ -673,18 +932,20 @@ mod test {
 
         map.with_channels(|map| {
             map.insert(closed("machen"));
-            map.insert(ch("feinen"));
             map.insert(closed("wir"));
+            map.insert(ch("wir"));
+            map.insert(ch("feinen"));
+            map.insert(ch("Fug"));
             map.insert(ch("Fug"));
         })?;
 
         map.remove_unusable().unwrap();
 
         map.with_channels(|map| {
-            assert!(map.by_id(&str_to_ed("m")).is_none());
-            assert!(map.by_id(&str_to_ed("w")).is_none());
-            assert!(map.by_id(&str_to_ed("f")).is_some());
-            assert!(map.by_id(&str_to_ed("F")).is_some());
+            assert_eq!(map.by_id(&str_to_ed("m")).len(), 0);
+            assert_eq!(map.by_id(&str_to_ed("w")).len(), 1);
+            assert_eq!(map.by_id(&str_to_ed("f")).len(), 1);
+            assert_eq!(map.by_id(&str_to_ed("F")).len(), 2);
         })?;
 
         Ok(())
@@ -720,8 +981,8 @@ mod test {
 
         let with_ch = |f: &dyn Fn(&FakeChannel)| {
             let inner = map.inner.lock().unwrap();
-            let ch = inner.channels.by_ed25519(&str_to_ed("t"));
-            let ch = ch.unwrap().unwrap_open();
+            let mut ch = inner.channels.by_ed25519(&str_to_ed("t"));
+            let ch = ch.next().unwrap().unwrap_open();
             f(ch);
         };
 
@@ -757,13 +1018,13 @@ mod test {
                 "wello",
                 Duration::from_secs(180),
                 Some(181),
-            ))
+            ));
         })?;
 
         // Minimum value of max unused duration is 180 seconds
         assert_eq!(180, map.expire_channels().as_secs());
         map.with_channels(|map| {
-            assert!(map.by_ed25519(&str_to_ed("w")).is_none());
+            assert_eq!(map.by_ed25519(&str_to_ed("w")).len(), 0);
         })?;
 
         let map = new_test_state();
@@ -796,10 +1057,10 @@ mod test {
         // Return duration until next channel expires
         assert_eq!(10, map.expire_channels().as_secs());
         map.with_channels(|map| {
-            assert!(map.by_ed25519(&str_to_ed("w")).is_some());
-            assert!(map.by_ed25519(&str_to_ed("y")).is_some());
-            assert!(map.by_ed25519(&str_to_ed("h")).is_some());
-            assert!(map.by_ed25519(&str_to_ed("g")).is_none());
+            assert_eq!(map.by_ed25519(&str_to_ed("w")).len(), 1);
+            assert_eq!(map.by_ed25519(&str_to_ed("y")).len(), 1);
+            assert_eq!(map.by_ed25519(&str_to_ed("h")).len(), 1);
+            assert_eq!(map.by_ed25519(&str_to_ed("g")).len(), 0);
         })?;
         Ok(())
     }
