@@ -41,6 +41,12 @@
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
+// TODO #1645 (either remove this, or decide to have it everywhere)
+#![cfg_attr(
+    not(all(feature = "full", feature = "experimental")),
+    allow(unused, unreachable_pub)
+)]
+
 #[macro_use] // SerdeStringOrTransparent
 mod time_store;
 
@@ -56,7 +62,6 @@ mod ipt_mgr;
 mod ipt_set;
 mod keys;
 mod netdir;
-mod nickname;
 mod publish;
 mod rend_handshake;
 mod replay;
@@ -95,10 +100,10 @@ pub use keys::{
     BlindIdKeypairSpecifier, BlindIdPublicKeySpecifier, DescSigningKeypairSpecifier,
     HsIdKeypairSpecifier, HsIdPublicKeySpecifier,
 };
-pub use nickname::{HsNickname, InvalidNickname};
 pub use publish::UploadError as DescUploadError;
 pub use req::{RendRequest, StreamRequest};
 pub use tor_hscrypto::pk::HsId;
+pub use tor_persist::hsnickname::{HsNickname, InvalidNickname};
 
 pub use helpers::handle_rend_requests;
 
@@ -286,7 +291,10 @@ impl OnionService {
             .storage_handle("iptpub")
             .map_err(StartupError::StateDirectoryInaccessible)?;
 
-        let (rend_req_tx, rend_req_rx) = mpsc::channel(32);
+        // If the HS implementation is stalled somehow, this is a local problem.
+        // We shouldn't kill the HS even if this is the oldest data in the system.
+        let (rend_req_tx, rend_req_rx) = mpsc_channel_no_memquota(32);
+
         let (shutdown_tx, shutdown_rx) = broadcast::channel(0);
         let (config_tx, config_rx) = postage::watch::channel_with(Arc::new(config));
 
