@@ -1,4 +1,7 @@
 //! Publish and maintain onion service descriptors
+//!
+//! See the [`reactor`] module-level documentation for more details.
+
 mod backoff;
 mod descriptor;
 mod reactor;
@@ -120,23 +123,6 @@ impl<R: Runtime, M: Mockable> Publisher<R, M> {
     }
 }
 
-//
-// Our main loop has to look something like:
-
-// Whenever time period or keys or netdir changes: Check whether our list of
-// HsDirs has changed.  If it is, add and/or remove hsdirs as needed.
-
-// "when learning about new keys, new intro points, or new configurations,
-// or whenever the time period changes: Mark descriptors dirty."
-
-// Whenever descriptors are dirty, we have enough info to generate
-// descriptors, and we aren't upload-rate-limited: Generate new descriptors
-// and mark descriptors clean.  Mark all hsdirs as needing new versions of
-// this descriptor.
-
-// While any hsdir does not have the latest version of its any descriptor:
-// upload it.  Retry with usual timeouts on failure."
-
 #[cfg(test)]
 mod test {
     // @@ begin test lint list maintained by maint/add_warning @@
@@ -172,7 +158,8 @@ mod test {
     use tor_basic_utils::test_rng::{testing_rng, TestingRng};
     use tor_circmgr::hspool::HsCircKind;
     use tor_hscrypto::pk::{HsBlindId, HsDescSigningKeypair, HsId, HsIdKey, HsIdKeypair};
-    use tor_keymgr::{ArtiNativeKeystore, KeyMgrBuilder, KeySpecifier, ToEncodableKey};
+    use tor_key_forge::ToEncodableKey;
+    use tor_keymgr::{ArtiNativeKeystore, KeyMgrBuilder, KeySpecifier};
     use tor_llcrypto::pk::{ed25519, rsa};
     use tor_netdir::testprovider::TestNetDirProvider;
     use tor_netdir::{testnet, NetDir};
@@ -185,7 +172,7 @@ mod test {
     use crate::publish::reactor::MockableClientCirc;
     use crate::status::{OnionServiceStatus, StatusSender};
     use crate::test::create_storage_handles;
-    use crate::{Anonymity, HsNickname};
+    use crate::HsNickname;
     use crate::{
         BlindIdKeypairSpecifier, BlindIdPublicKeySpecifier, DescSigningKeypairSpecifier,
         HsIdKeypairSpecifier, HsIdPublicKeySpecifier,
@@ -365,7 +352,12 @@ mod test {
         K: ToEncodableKey,
     {
         keymgr
-            .insert(key, svc_key_spec, tor_keymgr::KeystoreSelector::Default)
+            .insert(
+                key,
+                svc_key_spec,
+                tor_keymgr::KeystoreSelector::Primary,
+                true,
+            )
             .unwrap();
     }
 
@@ -393,7 +385,7 @@ mod test {
 
         // Provision the keystore with the necessary keys:
         let keymgr = KeyMgrBuilder::default()
-            .default_store(Box::new(keystore))
+            .primary_store(Box::new(keystore))
             .build()
             .unwrap();
 
@@ -434,7 +426,6 @@ mod test {
     fn build_test_config(nickname: HsNickname) -> OnionServiceConfig {
         OnionServiceConfigBuilder::default()
             .nickname(nickname)
-            .anonymity(Anonymity::Anonymous)
             .rate_limit_at_intro(None)
             .build()
             .unwrap()
@@ -453,6 +444,7 @@ mod test {
         poll_read_responses: I,
         expected_upload_count: usize,
         republish_count: usize,
+        expect_errors: bool,
     ) {
         runtime.clone().block_on(async move {
             let netdir_provider: Arc<dyn NetDirProvider> =
@@ -464,6 +456,7 @@ mod test {
                 responses_for_hsdir: Arc::new(Mutex::new(Default::default())),
             };
 
+            let mut status_rx = status_tx.subscribe();
             let publisher: Publisher<MockRuntime, MockReactorState<_>> = Publisher::new(
                 runtime.clone(),
                 nickname,
@@ -477,6 +470,9 @@ mod test {
 
             publisher.launch().unwrap();
             runtime.progress_until_stalled().await;
+            let status = status_rx.next().await.unwrap().publisher_status();
+            assert_eq!(State::Shutdown, status.state());
+            assert!(status.current_problem().is_none());
 
             // Check that we haven't published anything yet
             assert_eq!(publish_count.load(Ordering::SeqCst), 0);
@@ -484,6 +480,7 @@ mod test {
             reactor_event();
 
             runtime.progress_until_stalled().await;
+
             // We need to manually advance the time, because some of our tests check that the
             // failed uploads are retried, and there's a sleep() between the retries
             // (see BackoffSchedule::next_delay).
@@ -493,6 +490,17 @@ mod test {
             let initial_publish_count = publish_count.load(Ordering::SeqCst);
             assert_eq!(initial_publish_count, expected_upload_count);
 
+            let status = status_rx.next().await.unwrap().publisher_status();
+            if expect_errors {
+                // The upload results aren't ready yet.
+                assert_eq!(State::Bootstrapping, status.state());
+            } else {
+                // The test network doesn't have an SRV for the previous TP,
+                // so we are "unreachable".
+                assert_eq!(State::DegradedUnreachable, status.state());
+            }
+            assert!(status.current_problem().is_none());
+
             if republish_count > 0 {
                 /// The latest time the descriptor can be republished.
                 const MAX_TIMEOUT: Duration = Duration::from_secs(60 * 120);
@@ -531,6 +539,7 @@ mod test {
         poll_read_responses: I,
         multiplier: usize,
         republish_count: usize,
+        expect_errors: bool,
     ) {
         let runtime = MockRuntime::new();
         let nickname = HsNickname::try_from(TEST_SVC_NICKNAME.to_string()).unwrap();
@@ -586,6 +595,7 @@ mod test {
             poll_read_responses,
             expected_upload_count,
             republish_count,
+            expect_errors,
         );
     }
 
@@ -594,7 +604,7 @@ mod test {
         // The HSDirs always respond with 200 OK, so we expect to publish hsdir_count times.
         let poll_reads = [Ok(OK_RESPONSE.into())].into_iter();
 
-        test_temp_dir!().used_by(|dir| publish_after_ipt_change(dir, poll_reads, 1, 0));
+        test_temp_dir!().used_by(|dir| publish_after_ipt_change(dir, poll_reads, 1, 0, false));
     }
 
     #[test]
@@ -618,7 +628,7 @@ mod test {
             ]
             .into_iter();
 
-            test_temp_dir!().used_by(|dir| publish_after_ipt_change(dir, poll_reads, 2, 0));
+            test_temp_dir!().used_by(|dir| publish_after_ipt_change(dir, poll_reads, 2, 0, true));
         }
     }
 
@@ -629,7 +639,7 @@ mod test {
         const REUPLOAD_COUNT: usize = 4;
 
         test_temp_dir!()
-            .used_by(|dir| publish_after_ipt_change(dir, poll_reads, 1, REUPLOAD_COUNT));
+            .used_by(|dir| publish_after_ipt_change(dir, poll_reads, 1, REUPLOAD_COUNT, false));
     }
 
     // TODO (#1120): test that the descriptor is republished when the config changes
