@@ -82,7 +82,7 @@ impl HsDirParams {
         self.srv_lifespan.start
     }
 
-    /// Return an opaque offset for `when` from the start of the the shared-random-value protocol
+    /// Return an opaque offset for `when` from the start of the shared-random-value protocol
     /// period corresponding to the SRV for this time period.
     ///
     /// When uploading, callers should this offset to determine
@@ -286,6 +286,7 @@ mod test {
     #![allow(clippy::bool_assert_comparison)]
     #![allow(clippy::clone_on_copy)]
     #![allow(clippy::dbg_macro)]
+    #![allow(clippy::mixed_attributes_style)]
     #![allow(clippy::print_stderr)]
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
