@@ -60,16 +60,16 @@ struct HsDesc<'a> {
     intro_auth_key_cert_expiry: SystemTime,
     /// The expiration time of an introduction point encryption key certificate.
     intro_enc_key_cert_expiry: SystemTime,
-    /// The list of clients authorized to access the hidden service.
+    /// The list of clients authorized to discover the hidden service.
     ///
-    /// If `None`, client authentication is disabled.
-    /// If `Some(&[])`, client authorization is enabled,
+    /// If `None`, restricted discovery is disabled.
+    /// If `Some(&[])`, restricted discovery is enabled,
     /// but there will be no authorized clients.
     ///
-    /// If client authorization is disabled, the resulting middle document will contain a single
+    /// If restricted discovery is disabled, the resulting middle document will contain a single
     /// `auth-client` line populated with random values.
     ///
-    /// Client authorization is disabled by default.
+    /// Restricted discovery is disabled by default.
     #[builder(default)]
     auth_clients: Option<&'a [curve25519::PublicKey]>,
     /// The lifetime of this descriptor, in minutes.
@@ -85,7 +85,7 @@ struct HsDesc<'a> {
     subcredential: Subcredential,
 }
 
-/// Client authorization parameters.
+/// Restricted discovery parameters.
 #[derive(Debug)]
 pub(super) struct ClientAuth<'a> {
     /// An ephemeral x25519 keypair generated by the hidden service (`KP_hss_desc_enc`).
@@ -93,7 +93,7 @@ pub(super) struct ClientAuth<'a> {
     /// A new keypair MUST be generated every time a descriptor is encoded, or the descriptor
     /// encryption will not be secure.
     ephemeral_key: HsSvcDescEncKeypair,
-    /// The list of authorized clients.
+    /// The list of clients authorized to discover this service.
     auth_clients: &'a [curve25519::PublicKey],
     /// The `N_hs_desc_enc` descriptor_cookie key generated by the hidden service.
     ///
@@ -112,7 +112,7 @@ impl<'a> ClientAuth<'a> {
         rng: &mut R,
     ) -> Option<ClientAuth<'a>> {
         let Some(auth_clients) = auth_clients else {
-            // Client auth is disabled
+            // Restricted discovery is disabled
             return None;
         };
 
@@ -417,7 +417,7 @@ mod test {
             .subcredential(subcredential);
 
         // Build and encode a new descriptor (cloning `builder` because it's needed later, when we
-        // test if client auth works):
+        // test if restricted discovery works):
         let encoded_desc = builder
             .clone()
             .build_sign(&mut Config::Deterministic.into_rng())
@@ -428,7 +428,7 @@ mod test {
             encoded_desc.as_str(),
             *blinded_id.as_ref().public(),
             &subcredential,
-            None, /* No client auth */
+            None, /* No restricted discovery */
         );
 
         let hs_desc_sign_cert =
@@ -455,7 +455,8 @@ mod test {
 
         assert_eq!(&*encoded_desc, &*reencoded_desc);
 
-        // The same test, this time with client auth enabled (with a single authorized client):
+        // The same test, this time with restricted discovery enabled
+        // (with a single authorized client):
         let client_kp: HsClientDescEncKeypair = HsClientDescEncKeypair::generate(&mut rng);
         let client_pkey = client_kp.public().as_ref();
         let auth_clients = vec![*client_pkey];
@@ -470,7 +471,7 @@ mod test {
             encoded_desc.as_str(),
             *blinded_id.as_ref().public(),
             &subcredential,
-            Some(&client_kp), /* With client auth */
+            Some(&client_kp), /* With restricted discovery */
         );
 
         let hs_desc_sign_cert =
