@@ -32,6 +32,7 @@ use educe::Educe;
 use crate::circuit::ClientCirc;
 
 use crate::circuit::StreamTarget;
+use crate::memquota::StreamAccount;
 use crate::stream::StreamReader;
 use tor_basic_utils::skip_fmt;
 use tor_cell::relaycell::msg::Data;
@@ -164,6 +165,11 @@ pub struct DataStreamCtrl {
     /// similar, if and when it stops moving around.
     #[cfg(feature = "stream-ctrl")]
     status: Arc<Mutex<DataStreamStatus>>,
+
+    /// The memory quota account that should be used for this stream's data
+    ///
+    /// Exists to keep the account alive
+    _memquota: StreamAccount,
 }
 
 /// The write half of a [`DataStream`], implementing [`futures::io::AsyncWrite`].
@@ -208,6 +214,12 @@ pub struct DataWriter {
     /// and we should refactor if so.
     state: Option<DataWriterState>,
 
+    /// The memory quota account that should be used for this stream's data
+    ///
+    /// Exists to keep the account alive
+    // If we liked, we could make this conditional; see DataReader.memquota
+    _memquota: StreamAccount,
+
     /// A control object that can be used to monitor and control this stream
     /// without needing to own it.
     #[cfg(feature = "stream-ctrl")]
@@ -238,6 +250,13 @@ pub struct DataReader {
     /// should refactor if so.
     state: Option<DataReaderState>,
 
+    /// The memory quota account that should be used for this stream's data
+    ///
+    /// Exists to keep the account alive
+    // If we liked, we could make this conditional on not(cfg(feature = "stream-ctrl"))
+    // since, DataStreamCtrl contains a StreamAccount clone too.  But that seems fragile.
+    _memquota: StreamAccount,
+
     /// A control object that can be used to monitor and control this stream
     /// without needing to own it.
     #[cfg(feature = "stream-ctrl")]
@@ -339,8 +358,8 @@ impl DataStream {
     ///
     /// For non-optimistic stream, function `wait_for_connection`
     /// must be called after to make sure CONNECTED is received.
-    pub(crate) fn new(reader: StreamReader, target: StreamTarget) -> Self {
-        Self::new_inner(reader, target, false)
+    pub(crate) fn new(reader: StreamReader, target: StreamTarget, memquota: StreamAccount) -> Self {
+        Self::new_inner(reader, target, false, memquota)
     }
 
     /// Wrap raw stream reader and target parts as a connected DataStream.
@@ -350,12 +369,21 @@ impl DataStream {
     ///
     /// This is used by hidden services, exit relays, and directory servers to accept streams.
     #[cfg(feature = "hs-service")]
-    pub(crate) fn new_connected(reader: StreamReader, target: StreamTarget) -> Self {
-        Self::new_inner(reader, target, true)
+    pub(crate) fn new_connected(
+        reader: StreamReader,
+        target: StreamTarget,
+        memquota: StreamAccount,
+    ) -> Self {
+        Self::new_inner(reader, target, true, memquota)
     }
 
     /// The shared implementation of the `new*()` functions.
-    fn new_inner(reader: StreamReader, target: StreamTarget, connected: bool) -> Self {
+    fn new_inner(
+        reader: StreamReader,
+        target: StreamTarget,
+        connected: bool,
+        memquota: StreamAccount,
+    ) -> Self {
         #[cfg(feature = "stream-ctrl")]
         let status = {
             let mut data_stream_status = DataStreamStatus::default();
@@ -368,6 +396,7 @@ impl DataStream {
         let ctrl = Arc::new(DataStreamCtrl {
             circuit: Arc::downgrade(target.circuit()),
             status: status.clone(),
+            _memquota: memquota.clone(),
         });
         #[cfg(feature = "experimental-api")]
         let circuit = target.circuit().clone();
@@ -380,6 +409,7 @@ impl DataStream {
                 #[cfg(feature = "stream-ctrl")]
                 status: status.clone(),
             })),
+            _memquota: memquota.clone(),
             #[cfg(feature = "stream-ctrl")]
             ctrl: ctrl.clone(),
         };
@@ -391,6 +421,7 @@ impl DataStream {
                 #[cfg(feature = "stream-ctrl")]
                 status,
             })),
+            _memquota: memquota,
             #[cfg(feature = "stream-ctrl")]
             ctrl: ctrl.clone(),
         };
