@@ -49,7 +49,7 @@ mod obj;
 use std::{convert::Infallible, sync::Arc};
 
 pub use dispatch::{DispatchTable, InvokeError, UpdateSink};
-pub use err::RpcError;
+pub use err::{RpcError, RpcErrorKind};
 pub use method::{
     check_method_names, is_method_name, iter_method_names, DeserMethod, DynMethod,
     InvalidMethodName, Method, NoUpdates, RpcMethod,
@@ -94,14 +94,15 @@ pub enum LookupError {
     WrongType(ObjectId),
 }
 
-impl tor_error::HasKind for LookupError {
-    fn kind(&self) -> tor_error::ErrorKind {
-        use tor_error::ErrorKind as EK;
+impl From<LookupError> for RpcError {
+    fn from(err: LookupError) -> Self {
         use LookupError as E;
-        match self {
-            E::NoObject(_) => EK::RpcObjectNotFound,
-            E::WrongType(_) => EK::RpcInvalidRequest,
-        }
+        use RpcErrorKind as EK;
+        let kind = match &err {
+            E::NoObject(_) => EK::ObjectNotFound,
+            E::WrongType(_) => EK::InvalidRequest,
+        };
+        RpcError::new(err.to_string(), kind)
     }
 }
 
@@ -203,10 +204,18 @@ impl<T: Context> ContextExt for T {}
 /// on the dispatch table before invoking the method.
 pub fn invoke_rpc_method(
     ctx: Arc<dyn Context>,
+    obj_id: &ObjectId,
     obj: Arc<dyn Object>,
     method: Box<dyn DynMethod>,
     sink: dispatch::BoxedUpdateSink,
 ) -> Result<dispatch::RpcResultFuture, InvokeError> {
+    match method.invoke_without_dispatch(Arc::clone(&ctx), obj_id) {
+        Err(InvokeError::NoDispatchBypass) => {
+            // fall through
+        }
+        other => return other,
+    }
+
     let (obj, invocable) = ctx
         .dispatch_table()
         .read()
@@ -285,10 +294,16 @@ mod test {
     async fn invoke() {
         let ctx = Arc::new(Ctx::from(DispatchTable::from_inventory()));
         let discard = || Box::pin(futures::sink::drain().sink_err_into());
-        let r = invoke_rpc_method(ctx.clone(), Arc::new(Swan), Box::new(GetKids), discard())
-            .unwrap()
-            .await
-            .unwrap();
+        let r = invoke_rpc_method(
+            ctx.clone(),
+            &ObjectId::from("Odile"),
+            Arc::new(Swan),
+            Box::new(GetKids),
+            discard(),
+        )
+        .unwrap()
+        .await
+        .unwrap();
         assert_eq!(serde_json::to_string(&r).unwrap(), r#"{"v":"cygnets"}"#);
 
         let r = invoke_special_method(ctx, Arc::new(Swan), Box::new(GetKids))
