@@ -9,7 +9,7 @@ use std::io::{self, Write as _};
 use std::iter;
 use std::mem;
 use std::pin::Pin;
-use std::sync::{Arc, Mutex, MutexGuard};
+use std::sync::{Arc, Mutex, MutexGuard, Weak};
 use std::task::{Context, Poll, RawWaker, RawWakerVTable, Waker};
 
 use futures::pin_mut;
@@ -19,7 +19,7 @@ use futures::FutureExt as _;
 use educe::Educe;
 use itertools::Either;
 use itertools::{chain, izip};
-use slotmap::DenseSlotMap;
+use slotmap_careful::DenseSlotMap;
 use std::backtrace::Backtrace;
 use strum::EnumIter;
 use tracing::trace;
@@ -75,7 +75,7 @@ struct ArcMutexData(Arc<Mutex<Data>>);
 
 /// Task id, module to hide `Ti` alias
 mod task_id {
-    slotmap::new_key_type! {
+    slotmap_careful::new_key_type! {
         /// Task ID, usually called `TaskId`
         ///
         /// Short name in special `task_id` module so that [`Debug`] is nice
@@ -193,7 +193,11 @@ enum TaskState {
 /// Consequently, a `Waker` mustn't either.
 struct ActualWaker {
     /// Executor state
-    data: ArcMutexData,
+    ///
+    /// The Waker musn't to hold a strong reference to the executor,
+    /// since typically a task holds a future that holds a Waker,
+    /// and the exeuctor holds the task - so that would be a cycle.
+    data: Weak<Mutex<Data>>,
 
     /// Which task this is
     id: TaskId,
@@ -475,7 +479,7 @@ impl MockExecutor {
 
             // Poll the selected task
             let waker = ActualWaker {
-                data: self.data.clone(),
+                data: Arc::downgrade(&self.data.0),
                 id,
             }
             .new_waker();
@@ -534,11 +538,20 @@ impl Data {
 }
 
 impl ActualWaker {
+    /// Obtain a strong reference to the executor's data
+    fn upgrade_data(&self) -> Option<ArcMutexData> {
+        Some(ArcMutexData(self.data.upgrade()?))
+    }
+
     /// Wake the task corresponding to this `ActualWaker`
     ///
     /// This is like `<Self as std::task::Wake>::wake()` but takes `&self`, not `Arc`
     fn wake(&self) {
-        let mut data = self.data.lock();
+        let Some(data) = self.upgrade_data() else {
+            // The executor is gone!  Don't try to wake.
+            return;
+        };
+        let mut data = data.lock();
         trace!("MockExecutor {:?} wake", &self.id);
         let Some(task) = data.tasks.get_mut(self.id) else {
             return;
@@ -713,12 +726,16 @@ static RAW_WAKER_VTABLE: RawWakerVTable = RawWakerVTable::new(
 //---------- Sleep location tracking and dumping ----------
 
 /// We record "where a future went to sleep" as (just) a backtrace
+///
+/// This type alias allows us to mock `Backtrace` for miri.
+/// (It also insulates from future choices about sleep location representation.0
+#[cfg(not(miri))]
 type SleepLocation = Backtrace;
 
 impl Data {
     /// Dump tasks and their sleep location backtraces
     fn dump_backtraces(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        for (id, task) in &self.tasks {
+        for (id, task) in self.tasks.iter() {
             let prefix = |f: &mut fmt::Formatter| write!(f, "{id:?}={task:?}: ");
             match &task.state {
                 Awake => {
@@ -753,8 +770,9 @@ impl Clone for ActualWaker {
     fn clone(&self) -> Self {
         let id = self.id;
 
-        {
-            let mut data = self.data.lock();
+        if let Some(data) = self.upgrade_data() {
+            // If the executor is gone, there is nothing to adjust
+            let mut data = data.lock();
             if let Some(task) = data.tasks.get_mut(self.id) {
                 match &mut task.state {
                     Awake => trace!("MockExecutor cloned waker for awake task {id:?}"),
@@ -950,6 +968,24 @@ impl Debug for Data {
     }
 }
 
+/// Mock `Backtrace` for miri
+///
+/// See also the not-miri `type SleepLocation`, alias above.
+#[cfg(miri)]
+mod miri_sleep_location {
+    #[derive(Debug, derive_more::Display)]
+    #[display("<SleepLocation>")]
+    pub(super) struct SleepLocation {}
+
+    impl SleepLocation {
+        pub(super) fn force_capture() -> Self {
+            SleepLocation {}
+        }
+    }
+}
+#[cfg(miri)]
+use miri_sleep_location::SleepLocation;
+
 #[cfg(test)]
 mod test {
     // @@ begin test lint list maintained by maint/add_warning @@
